@@ -29,6 +29,8 @@ T2 = TypeVar("T2", bound=Union[ConstType, "Expr"])
 V = TypeVar("V", bound=ConstType)
 
 if TYPE_CHECKING:
+    from types import CodeType
+
     from pydantic.annotated_handlers import GetCoreSchemaHandler
     from pydantic_core import core_schema
 
@@ -86,7 +88,7 @@ def safe_eval(expr: str | bool | Expr, context: Mapping | None = None) -> Any:
     """
     if isinstance(expr, bool):
         return expr
-    return parse_expression(expr).eval(context or {})
+    return parse_expression(expr).eval(context)
 
 
 class Expr(ast.AST, Generic[T]):
@@ -151,6 +153,9 @@ class Expr(ast.AST, Generic[T]):
     >>> new_expr.eval(dict(v2="hello!", myvar=8))
     'hello!'
 
+    you can also use keyword arguments.  This is *slightly* slower
+    >>> new_expr.eval(v2="hello!", myvar=4)
+
     serialize
     >>> str(new_expr)
     'myvar > 5 and v2'
@@ -172,21 +177,32 @@ class Expr(ast.AST, Generic[T]):
     >>> reveal_type(is_ready())  # revealed type is `bool`
     """
 
+    _names: set[str]
+    _code: CodeType
+
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         if type(self).__name__ == "Expr":
             raise RuntimeError("Don't instantiate Expr. Use `Expr.parse`")
         super().__init__(*args, **kwargs)
-        ast.fix_missing_locations(self)
+        self._recompile()
 
-    def eval(self, context: Mapping[str, object] | None = None) -> T:
+    def _recompile(self) -> None:
+        ast.fix_missing_locations(self)
+        self._code = compile(ast.Expression(body=self), "<Expr>", "eval")
+        self._names = set(self._iter_names())
+
+    def eval(
+        self, context: Mapping[str, object] | None = None, **ctx_kwargs: object
+    ) -> T:
         """Evaluate this expression with names in `context`."""
         if context is None:
-            context = {}
-        code = compile(ast.Expression(body=self), "<Expr>", "eval")
+            context = ctx_kwargs
+        elif ctx_kwargs:
+            context = {**context, **ctx_kwargs}
         try:
-            return cast(T, eval(code, {}, context))
+            return eval(self._code, {}, context)  # type: ignore
         except NameError as e:
-            miss = {k for k in _iter_names(self) if k not in context}
+            miss = {k for k in self._names if k not in context}
             raise NameError(
                 f"Names required to eval this expression are missing: {miss}"
             ) from e
@@ -345,6 +361,9 @@ class Expr(ast.AST, Generic[T]):
             _hash += hash(field)
         return _hash
 
+    def _iter_names(self) -> Iterator[str]:
+        yield from _iter_names(self)
+
 
 LOAD = ast.Load()
 
@@ -359,12 +378,6 @@ class Name(Expr[T], ast.Name):
         kwargs["ctx"] = LOAD
         super().__init__(id, **kwargs)
 
-    def eval(self, context: Mapping | None = None) -> T:
-        """Evaluate this expression with names in `context`."""
-        if context is None:
-            context = {}
-        return super().eval(context=context)
-
 
 class Constant(Expr[V], ast.Constant):
     """A constant value.
@@ -462,6 +475,45 @@ class IfExp(Expr, ast.IfExp):
         )
 
 
+class Tuple(Expr, ast.Tuple):
+    """A tuple expression.
+
+    `elts` is a list of expressions.
+    """
+
+    def __init__(
+        self, elts: Sequence[Expr], ctx: ast.expr_context = LOAD, **kwargs: Any
+    ) -> None:
+        kwargs["ctx"] = ctx
+        super().__init__(elts=[Expr._cast(e) for e in elts], **kwargs)
+
+
+class List(Expr, ast.List):
+    """A tuple expression.
+
+    `elts` is a list of expressions.
+    """
+
+    def __init__(
+        self, elts: Sequence[Expr], ctx: ast.expr_context = LOAD, **kwargs: Any
+    ) -> None:
+        kwargs["ctx"] = ctx
+        super().__init__(elts=[Expr._cast(e) for e in elts], **kwargs)
+
+
+class Set(Expr, ast.Set):
+    """A tuple expression.
+
+    `elts` is a list of expressions.
+    """
+
+    def __init__(
+        self, elts: Sequence[Expr], ctx: ast.expr_context = LOAD, **kwargs: Any
+    ) -> None:
+        kwargs["ctx"] = ctx
+        super().__init__(elts=[Expr._cast(e) for e in elts], **kwargs)
+
+
 class ExprTransformer(ast.NodeTransformer):
     """Transformer that converts an ast.expr into an `Expr`.
 
@@ -555,6 +607,15 @@ class _ExprSerializer(ast.NodeVisitor):
     def visit_Name(self, node: ast.Name) -> None:
         self.write(node.id)
 
+    def visit_Tuple(self, node: ast.Tuple) -> None:
+        self.write(f'({", ".join(map(str, node.elts))})')
+
+    def visit_Set(self, node: ast.Set) -> None:
+        self.write("{" + ", ".join(map(str, node.elts)) + "}")
+
+    def visit_List(self, node: ast.List) -> None:
+        self.write(f'[{", ".join(map(str, node.elts))}]')
+
     def visit_ContextKey(self, node: ContextKey) -> None:
         return self.visit_Name(node)
 
